@@ -1,7 +1,7 @@
 ### Plots for mvr objects.  Some of them also work for other
 ### objects, but that is not a priority.
 ###
-### $Id: plots.R 109 2007-04-19 12:15:08Z bhm $
+### $Id: plots.R 148 2007-10-16 20:33:38Z bhm $
 
 ###
 ### Plot method for mvr objects
@@ -104,7 +104,9 @@ loadingplot.default <- function(object, comps = 1:2, scatter = FALSE, labels,
     ## Check arguments
     nComps <- length(comps)
     if (nComps == 0) stop("At least one component must be selected.")
-    if (!missing(type) && sum(nchar(type)) != 1) stop("Invalid plot type.")
+    if (!missing(type) &&
+        (length(type) != 1 || is.na(nchar(type, "c")) || nchar(type, "c") != 1))
+        stop("Invalid plot type.")
     ## Get the loadings
     if (is.matrix(object)) {
         ## Assume this is already a loading matrix
@@ -123,7 +125,12 @@ loadingplot.default <- function(object, comps = 1:2, scatter = FALSE, labels,
             ## Set up point/tick mark labels
             if (length(labels) == 1) {
                 labels <- switch(match.arg(labels, c("names", "numbers")),
-                                 names = rownames(L),
+                                 names = {
+                                     if (is.null(rnames <- rownames(L))) {
+                                         stop("The loadings have no row names.")
+                                     } else {
+                                         rnames
+                                     }},
                                  numbers = 1:nrow(L)
                                  )
             }
@@ -189,7 +196,7 @@ loadingplot.default <- function(object, comps = 1:2, scatter = FALSE, labels,
                                    xaxt <- par("xaxt")
                                }
                            } else {
-                               stop("Could not convert variable names to numbers")
+                               stop("Could not convert variable names to numbers.")
                            }
                        }
                        )
@@ -246,7 +253,7 @@ corrplot <- function(object, comps = 1:2, labels, radii = c(sqrt(1/2), 1),
     } else {
         S <- scores(object)[,comps, drop = FALSE]
         if (is.null(S))
-            stop("`", deparse(substitute(object)), "' has no scores")
+            stop("`", deparse(substitute(object)), "' has no scores.")
         cl <- cor(model.matrix(object), S)
         varlab <- compnames(object, comps, explvar = TRUE)
     }
@@ -324,7 +331,7 @@ predplot.default <- function(object, ...) {
 ## Method for mvr objects:
 predplot.mvr <- function(object, ncomp = object$ncomp, which, newdata,
                          nCols, nRows, xlab = "measured", ylab = "predicted",
-                         ..., font.main = 1, cex.main = 1.1)
+                         main, ..., font.main, cex.main)
 {
     ## Select type(s) of prediction
     if (missing(which)) {
@@ -355,7 +362,11 @@ predplot.mvr <- function(object, ncomp = object$ncomp, which, newdata,
     ## Set plot parametres as needed:
     dims <- c(nEst, nSize, nResp)
     dims <- dims[dims > 1]
-    if (length(dims) > 0) {
+    nPlots <- prod(dims)
+    if (nPlots > 1) {
+        ## Set up default font.main and cex.main for individual titles:
+        if (missing(font.main)) font.main <- 1
+        if (missing(cex.main)) cex.main <- 1.1
         ## Show the *labs in the margin:
         mXlab <- xlab
         mYlab <- ylab
@@ -366,9 +377,13 @@ predplot.mvr <- function(object, ncomp = object$ncomp, which, newdata,
         opar <- par(no.readonly = TRUE)
         on.exit(par(opar))
         par(mfrow = c(nRows, nCols),
-            oma = c(1,1,0,0) + 0.1, mar = c(3,3,3,1) + 0.1)
-        if (nRows * nCols < prod(dims)) par(ask = TRUE)
+            oma = c(1, 1, if(missing(main)) 0 else 2, 0) + 0.1,
+            mar = c(3,3,3,1) + 0.1)
+        if (nRows * nCols < nPlots && dev.interactive()) par(ask = TRUE)
     } else {
+        ## Set up default font.main and cex.main for the main title:
+        if (missing(font.main)) font.main <- par("font.main")
+        if (missing(cex.main)) cex.main <- par("cex.main")
         nCols <- nRows <- 1
     }
 
@@ -397,8 +412,13 @@ predplot.mvr <- function(object, ncomp = object$ncomp, which, newdata,
         for (size in 1:nSize) {
             for (est in 1:nEst) {
                 plotNo <- plotNo + 1
-                main <- sprintf("%s, %d comps, %s", respnames(object)[resp],
-                                ncomp[size], which[est])
+                if (nPlots == 1 && !missing(main)) {
+                    lmain <- main
+                } else {
+                    lmain <- sprintf("%s, %d comps, %s",
+                                     respnames(object)[resp],
+                                     ncomp[size], which[est])
+                }
                 sub <- which[est]
                 switch(which[est],
                        train = {
@@ -414,18 +434,20 @@ predplot.mvr <- function(object, ncomp = object$ncomp, which, newdata,
                            predicted <- test.predicted[,resp,size]
                        }
                        )
-                if (length(dims) > 0 &&
-                    (plotNo %% (nCols * nRows) == 0 || plotNo == prod(dims))) {
-                    ## Last plot on a page; add outer margin text:
+                xy <- predplotXy(measured, predicted, main = lmain,
+                                 font.main = font.main, cex.main = cex.main,
+                                 xlab = xlab, ylab = ylab, ...)
+                if (nPlots > 1 &&
+                    (plotNo %% (nCols * nRows) == 0 || plotNo == nPlots)) {
+                    ## Last plot on a page; add outer margin text and title:
                     mtext(mXlab, side = 1, outer = TRUE)
                     mtext(mYlab, side = 2, outer = TRUE)
+                    if (!missing(main)) title(main = main, outer = TRUE)
                 }
-                predplotXy(measured, predicted, main = main,
-                           font.main = font.main, cex.main = cex.main,
-                           xlab = xlab, ylab = ylab, ...)
             }
         }
     }
+    invisible(xy)
 }
 
 ## The workhorse function:
@@ -449,7 +471,7 @@ coefplot <- function(object, ncomp = object$ncomp, comps, intercept = FALSE,
                      type = "l", lty = 1:nLines, lwd = NULL,
                      pch = 1:nLines, cex = NULL, col = 1:nLines, legendpos,
                      xlab = "variable", ylab = "regression coefficient",
-                     pretty.xlabels = TRUE, xlim, ...)
+                     main, pretty.xlabels = TRUE, xlim, ...)
 {
     ## This simplifies code below:
     if (missing(comps)) comps <- NULL
@@ -462,7 +484,8 @@ coefplot <- function(object, ncomp = object$ncomp, comps, intercept = FALSE,
     ## Set plot parametres as needed:
     dims <- c(nSize, nResp)
     dims <- dims[dims > 1]
-    if (length(dims) > 0) {
+    nPlots <- prod(dims)
+    if (nPlots > 1) {
         ## Show the *labs in the margin:
         mXlab <- xlab
         mYlab <- ylab
@@ -473,13 +496,15 @@ coefplot <- function(object, ncomp = object$ncomp, comps, intercept = FALSE,
         opar <- par(no.readonly = TRUE)
         on.exit(par(opar))
         par(mfrow = c(nRows, nCols),
-            oma = c(1,1,0,0) + 0.1, mar = c(3,3,3,1) + 0.1)
-        if (nRows * nCols < prod(dims)) par(ask = TRUE)
+            oma = c(1, 1, if(missing(main)) 0 else 2, 0) + 0.1,
+            mar = c(3,3,3,1) + 0.1)
+        if (nRows * nCols < nPlots && dev.interactive()) par(ask = TRUE)
     } else {
         nCols <- nRows <- 1
     }
     if (length(lty) > nLines) lty <- lty[1:nLines] # otherwise legend chokes
-    if (sum(nchar(type)) != 1) stop("invalid plot type")
+    if (length(type) != 1 || is.na(nchar(type, "c")) || nchar(type, "c") != 1)
+        stop("Invalid plot type.")
     ## Are we plotting lines?
     dolines <- type %in% c("l", "b", "c", "o", "s", "S", "h")
     ## Are we plotting points?
@@ -511,7 +536,7 @@ coefplot <- function(object, ncomp = object$ncomp, comps, intercept = FALSE,
                                xaxt <- par("xaxt")
                            }
                        } else {
-                           stop("Could not convert variable names to numbers")
+                           stop("Could not convert variable names to numbers.")
                        }
                    }
                    )
@@ -527,21 +552,20 @@ coefplot <- function(object, ncomp = object$ncomp, comps, intercept = FALSE,
         for (size in 1:nSize) {
             plotNo <- plotNo + 1
 
-            if (length(dims) > 0 &&
-                (plotNo %% (nCols * nRows) == 0 || plotNo == prod(dims))) {
-                ## Last plot on a page; add outer margin text:
-                mtext(mXlab, side = 1, outer = TRUE)
-                mtext(mYlab, side = 2, outer = TRUE)
+            if (nPlots == 1 && !missing(main)) {
+                lmain <- main
+            } else if (separate) {
+                lmain <- paste(respname, complabs[size], sep = ", ")
+            } else {
+                lmain <- respname
             }
-
             if (separate) {
                 plot(xnum, coefs[,resp,size],
-                     main = paste(respname, complabs[size], sep = ", "),
-                     xlab = xlab, ylab = ylab, type = type,
+                     main = lmain, xlab = xlab, ylab = ylab, type = type,
                      lty = lty, lwd = lwd, pch = pch, cex = cex,
                      col = col, xaxt = xaxt, xlim = xlim, ...)
             } else {
-                matplot(xnum, coefs[,resp,], main = respname, xlab = xlab,
+                matplot(xnum, coefs[,resp,], main = lmain, xlab = xlab,
                         ylab = ylab, type = type, lty = lty, lwd = lwd,
                         pch = pch, cex = cex, col = col, xaxt = xaxt,
                         xlim = xlim, ...)
@@ -563,6 +587,14 @@ coefplot <- function(object, ncomp = object$ncomp, comps, intercept = FALSE,
                 axis(1, ticks, labels[ticks], ...)
             }
             abline(h = 0, col = "gray")
+
+            if (nPlots > 1 &&
+                (plotNo %% (nCols * nRows) == 0 || plotNo == nPlots)) {
+                ## Last plot on a page; add outer margin text and title:
+                mtext(mXlab, side = 1, outer = TRUE)
+                mtext(mYlab, side = 2, outer = TRUE)
+                if (!missing(main)) title(main, outer = TRUE)
+            }
         }
     }
 }
@@ -585,7 +617,8 @@ validationplot <- function(object, val.type = c("RMSEP", "MSEP", "R2"),
 ## A plot method for mvrVal objects:
 plot.mvrVal <- function(x, nCols, nRows, type = "l", lty = 1:nEst, lwd = NULL,
                         pch = 1:nEst, cex = NULL, col = 1:nEst, legendpos,
-                        xlab = "number of components", ylab = x$type, ...)
+                        xlab = "number of components", ylab = x$type, main,
+                        ...)
 {
     if (!is.null(x$call$cumulative) && eval(x$call$cumulative) == FALSE)
         stop("`cumulative = FALSE' not supported.")
@@ -601,8 +634,9 @@ plot.mvrVal <- function(x, nCols, nRows, type = "l", lty = 1:nEst, lwd = NULL,
         opar <- par(no.readonly = TRUE)
         on.exit(par(opar))
         par(mfrow = c(nRows, nCols),
-            oma = c(1,1,0,0) + 0.1, mar = c(3,3,3,1) + 0.1)
-        if (nRows * nCols < nResp) par(ask = TRUE)
+            oma = c(1, 1, if(missing(main)) 0 else 2, 0) + 0.1,
+            mar = c(3,3,3,1) + 0.1)
+        if (nRows * nCols < nResp && dev.interactive()) par(ask = TRUE)
     } else {
         nCols <- nRows <- 1
     }
@@ -610,29 +644,28 @@ plot.mvrVal <- function(x, nCols, nRows, type = "l", lty = 1:nEst, lwd = NULL,
     estnames <- dimnames(x$val)[[1]]    # Names of estimators
     nEst <- length(estnames)
     if (length(lty) > nEst) lty <- lty[1:nEst] # otherwise legend chokes
-    if (sum(nchar(type)) != 1) stop("invalid plot type")
+    if (length(type) != 1 || is.na(nchar(type, "c")) || nchar(type, "c") != 1)
+        stop("Invalid plot type.")
     ## Are we plotting lines?
     dolines <- type %in% c("l", "b", "c", "o", "s", "S", "h")
     ## Are we plotting points?
     dopoints <- type %in% c("p", "b", "o")
 
     for (resp in 1:nResp) {
-        if (nResp > 1 && (resp %% (nCols * nRows) == 0 || resp == nResp)) {
-            ## Last plot on a page; add outer margin text:
-            mtext(mXlab, side = 1, outer = TRUE)
-            mtext(mYlab, side = 2, outer = TRUE)
+        if (nResp == 1 && !missing(main)) {
+            lmain <- main
+        } else {
+            lmain <- ynames[resp]
         }
-
         y <- x$val[,resp,]
         if (is.matrix(y)) y <- t(y)
-        if (identical(all.equal(x$comps, min(x$comps):max(x$comps)),
-                      TRUE)) {
-            matplot(x$comps, y, xlab = xlab, ylab = ylab, main = ynames[resp],
+        if (isTRUE(all.equal(x$comps, min(x$comps):max(x$comps)))) {
+            matplot(x$comps, y, xlab = xlab, ylab = ylab, main = lmain,
                     type = type, lty = lty, lwd = lwd, pch = pch, cex = cex,
                     col = col, ...)
         } else {
             ## Handle irregular x$comps:
-            matplot(y, xlab = xlab, ylab = ylab, main = ynames[resp],
+            matplot(y, xlab = xlab, ylab = ylab, main = lmain,
                     xaxt = "n", type = type, lty = lty, lwd = lwd,
                     pch = pch, cex = cex, col = col, ...)
             axis(1, seq(along = x$comps), x$comps)
@@ -643,6 +676,12 @@ plot.mvrVal <- function(x, nCols, nRows, type = "l", lty = 1:nEst, lwd = NULL,
                                 if (dopoints) list(pch = pch, pt.cex = cex,
                                                    pt.lwd = lwd)))
         }
+        if (nResp > 1 && (resp %% (nCols * nRows) == 0 || resp == nResp)) {
+            ## Last plot on a page; add outer margin text and title:
+            mtext(mXlab, side = 1, outer = TRUE)
+            mtext(mYlab, side = 2, outer = TRUE)
+            if (!missing(main)) title(main, outer = TRUE)
+        }
     }
 }
 
@@ -654,7 +693,7 @@ plot.mvrVal <- function(x, nCols, nRows, type = "l", lty = 1:nEst, lwd = NULL,
 biplot.mvr <- function(x, comps = 1:2,
                        which = c("x", "y", "scores", "loadings"),
                        var.axes = FALSE, xlabs, ylabs, main, ...) {
-    if (length(comps) != 2) stop("Exactly 2 components must be selected")
+    if (length(comps) != 2) stop("Exactly 2 components must be selected.")
     which <- match.arg(which)
     switch(which,
            x = {
@@ -679,7 +718,7 @@ biplot.mvr <- function(x, comps = 1:2,
            }
            )
     if (is.null(objects) || is.null(vars))
-        stop("'x' lacks the required scores/loadings")
+        stop("'x' lacks the required scores/loadings.")
     ## Build a call to `biplot'
     mc <- match.call()
     mc$comps <- mc$which <- NULL
