@@ -1,5 +1,5 @@
 ### summaries.R: print and summary methods.
-### $Id: summaries.R 48 2005-10-07 12:52:59Z bhm $
+### $Id: summaries.R 132 2007-08-24 09:21:05Z bhm $
 
 ## Print method for mvr objects:
 print.mvr <- function(x, ...) {
@@ -8,6 +8,10 @@ print.mvr <- function(x, ...) {
                regr = "Partial least squares"
                alg = "kernel"
            },
+           widekernelpls = {
+               regr = "Partial least squares"
+               alg = "wide kernel"
+           },
            simpls = {
                regr = "Partial least squares"
                alg = "simpls"
@@ -37,15 +41,15 @@ summary.mvr <- function(object, what = c("all", "validation", "training"),
     what <- match.arg(what)
     if (what == "all") what <- c("validation", "training")
     if (is.null(object$validation)) what <- "training"
-  
+
     nobj <- nrow(object$scores)
-    npred <- length(object$Ymeans)
+    nresp <- length(object$Ymeans)
     yvarnames <- respnames(object)
     cat("Data: \tX dimension:", nobj, length(object$Xmeans),
-        "\n\tY dimension:", nobj, npred)
+        "\n\tY dimension:", nobj, nresp)
     cat("\nFit method:", object$method)
     cat("\nNumber of components considered:", object$ncomp)
-  
+
     for (wh in what) {
         if (wh == "training") {
             cat("\nTRAINING: % variance explained\n")
@@ -67,11 +71,11 @@ summary.mvr <- function(object, what = c("all", "validation", "training"),
 
 ## Print method for mvrVal objects:
 print.mvrVal <- function(x, digits = 4, print.gap = 2, ...) {
-    npred <- dim(x$val)[2]
+    nresp <- dim(x$val)[2]
     yvarnames <- dimnames(x$val)[[2]]
     names(dimnames(x$val)) <- NULL
-    for (i in 1:npred) {
-        if (npred > 1) cat("\nResponse:", yvarnames[i], "\n")
+    for (i in 1:nresp) {
+        if (nresp > 1) cat("\nResponse:", yvarnames[i], "\n")
         print(x$val[,i,], digits = digits, print.gap = print.gap, ...)
     }
     invisible(x)
